@@ -5,6 +5,7 @@ import asyncio
 import logging
 import traceback
 import importlib
+from dataclasses import dataclass
 from types import ModuleType
 
 from starlette.status import WS_1001_GOING_AWAY
@@ -16,12 +17,11 @@ from starlette.middleware.trustedhost import TrustedHostMiddleware
 from jinja2 import Environment, ChoiceLoader, FileSystemLoader, PackageLoader
 from a2wsgi import WSGIMiddleware
 
-from .types import WSGIApp, Scope, Receive, Send, ASGIApp, Message, Literal, TypedDict
+from .types import WSGIApp, Scope, Receive, Send, ASGIApp, Message, Literal
 from .utils import cached_property
 from .config import Config
-from .routing import Router, BaseRoute, NoMatchFound
+from .routing.routes import Router, BaseRoute, NoMatchFound
 from .http import responses
-from .http.view import parse_params
 from .http.debug import ServerErrorMiddleware
 from .http.request import Request
 from .http.responses import (
@@ -95,18 +95,22 @@ class Lifespan:
         """
         message = await receive()
         assert message["type"] == "lifespan.startup"
-
         try:
             await self.startup()
         except BaseException:
             msg = traceback.format_exc()
             await send({"type": "lifespan.startup.failed", "message": msg})
             raise
-
         await send({"type": "lifespan.startup.complete"})
+
         message = await receive()
         assert message["type"] == "lifespan.shutdown"
-        await self.shutdown()
+        try:
+            await self.shutdown()
+        except BaseException:
+            msg = traceback.format_exc()
+            await send({"type": "lifespan.shutdown.failed", "message": msg})
+            raise
         await send({"type": "lifespan.shutdown.complete"})
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
@@ -125,7 +129,7 @@ class IndexFile:
         self.module_name = module_name
         self.allow_underline = allow_underline
         if basepath is not None:
-            self.basepath = basepath
+            self.__dict__["basepath"] = basepath
         logger.debug(f"Index File in module {module_name}, basepath: {basepath}")
 
     @cached_property
@@ -236,7 +240,7 @@ class IndexFile:
                 yield module, path
 
     async def http(self, scope: Scope, receive: Receive, send: Send) -> None:
-        request = scope["app"].factory_class["http"](scope, receive, send)
+        request = scope["app"].factory_class.http(scope, receive, send)
         pathlist = self._split_path(request.url.path)
 
         module = self.get_view(request.url.path)
@@ -257,7 +261,7 @@ class IndexFile:
         await response(scope, receive, send)
 
     async def websocket(self, scope: Scope, receive: Receive, send: Send) -> None:
-        websocket = scope["app"].factory_class["websocket"](
+        websocket = scope["app"].factory_class.websocket(
             scope, receive=receive, send=send
         )
         pathlist = self._split_path(websocket.url.path)
@@ -293,9 +297,10 @@ class IndexFile:
         await handler(scope, receive, send)
 
 
-FactoryClass = TypedDict(
-    "FactoryClass", {"http": typing.Type[Request], "websocket": typing.Type[WebSocket]},
-)
+@dataclass
+class FactoryClass:
+    http: typing.Type[Request] = Request
+    websocket: typing.Type[WebSocket] = WebSocket
 
 
 class Index:
@@ -308,7 +313,7 @@ class Index:
         on_startup: typing.List[typing.Callable] = [],
         on_shutdown: typing.List[typing.Callable] = [],
         routes: typing.List[BaseRoute] = [],
-        factory_class: FactoryClass = {"http": Request, "websocket": WebSocket},
+        factory_class: FactoryClass = FactoryClass(),
     ) -> None:
         self.factory_class = factory_class
         self.router = Router(routes)
@@ -486,22 +491,12 @@ class Index:
         handler: typing.Optional[ASGIApp] = None
 
         try:
-            path_params, endpoint = self.router.search(scope["type"], scope["path"])
+            path_params, handler = self.router.search(scope["type"], scope["path"])
             scope["path_params"] = path_params
-            request = self.factory_class[scope["type"]](scope, receive, send)  # type: ignore
-            get_handler = await parse_params(endpoint, request)
-
-            for middleware in getattr(endpoint, "middlewares", ()):
-                get_handler = middleware(get_handler)
-
-            if scope["type"] == "http":
-                handler = convert(await get_handler(request))
-            else:
-                handler = get_handler(request)
         except NoMatchFound:
-            if scope["type"] == "http" or self.try_html:
+            if scope["type"] == "http" and self.try_html:
                 # only html, no middleware/background tasks or other anything
-                handler = try_html(self.factory_class["http"](scope, receive, send))
+                handler = try_html(self.factory_class.http(scope, receive, send))
 
         if handler is None:
             if scope["type"] == "http":
